/*********************************************************************************
 *																				
 *   e-Mail Notify - Internet e-Mail Notifier
 *   Copyright (C) 1996-1998 Ludovic Dubost <ldubost@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 **********************************************************************************/



#include "stdafx.h"
#include "popclien.h"
#include "email.h"
#include "emaildoc.h"
#include <winsock.h>
#include <stdarg.h>


#include <iostream.h>
#include <fstream.h>

#include "mainfrm.h"
#include "debug.h"


char hostdata[MAXGETHOSTSTRUCT];



/* ======================================================================== */
/* Defines */
/* ======================================================================== */
#define BUFFERSIZE 8192
#define CRLF "\r\n"


/* ======================================================================== */
/* Global Variables */
/* ======================================================================== */
char *transport   = "tcp";
char *service     = "pop3";



CPOPClient::CPOPClient(unsigned settingnb)
{
 m_pSocket = NULL;
 bCanceling = FALSE;
}

CPOPClient::~CPOPClient()
{
};


int CPOPClient::StopUpdate()
{
  bCanceling = TRUE;
  if (m_pSocket) m_pSocket->CancelBlockingCall();
  return 1;
}



HANDLE CPOPClient::SendAsyncDNS(HWND hwnd)
{
 DWORD dwAddr;
 if ((dwAddr = inet_addr(AfxGetConfig()->m_csPOPServer))==INADDR_NONE)
	return WSAAsyncGetHostByName(hwnd, WM_ASYNC_DNS,AfxGetConfig()->m_csPOPServer,hostdata,MAXGETHOSTSTRUCT);
 else if (dwAddr==NULL)
 {
  return (HANDLE)0;
 }
 else
 {
  struct hostent* hp = (hostent*)hostdata; 
  hp->h_addrtype = AF_INET;
  hp->h_length = 4;
  hp->h_addr_list = (char**)&(hp->h_addr_list) + 4;
  hp->h_addr_list[0] = (char*)(hp->h_addr_list) + 4;
  memcpy(hp->h_addr_list[0],&dwAddr,4);
  AfxGetMainFrame()->PostMessage(WM_ASYNC_DNS);	 
  return (HANDLE)1;
  // return WSAAsyncGetHostByAddr(hwnd, WM_ASYNC_DNS,(const char*)&dwAddr,4,PF_INET,hostdata,MAXGETHOSTSTRUCT);
 }
}


// Connexion: return -1 if no connexion possible
// or connexion canceled !
// and the number of mail received on success
int CPOPClient::POPConnect()
{
 int ErrorCode = 0;

 
 if ((AfxGetConfig()->m_csPOPServer=="")
     ||(AfxGetConfig()->m_csPOPUsername=="")
     ||(AfxGetConfig()->m_csPOPPassword==""))
  {
	SocketError(IDS_ERROR_CONFIG,"");
	return -1;
  }

 if (bCanceling) return -1;

 // Check to see if there is not something still there !
 if (m_pSocket)
  {
   TRACE("Still a socket object not deleted");
   delete m_pSocket;
   m_pSocket = NULL;
  }
 
 m_pSocket = new CPOPSocket();
 
 if (!m_pSocket->Create())
  {
	SocketError(IDS_ERROR_CREATESOCKET,"");
	return -1;
  }

 if (bCanceling) return -1;

 struct servent* sp;
 struct sockaddr_in raddr;
 struct hostent* hp = (hostent*)hostdata; 

 if (sp = getservbyname(service, transport))
    {
     raddr.sin_port   = sp->s_port;
    }
 else
    {
     raddr.sin_port   = 110;
	}

 raddr.sin_family = hp->h_addrtype;
 memcpy(&raddr.sin_addr,hp->h_addr_list[0], hp->h_length);

 Log("Trying to connect");
 if (!m_pSocket->Connect((struct sockaddr *)&raddr,sizeof(raddr)))
  {
    DWORD dwResult = WSAGetLastError();
	SocketError(IDS_ERROR_CONNECT,"");
	return -1;
  }
 if (bCanceling) return -1;

 CString strReplyConnect = POPReply(ErrorCode);
 if (strReplyConnect.Left(3)!="+OK")
  {
   SocketError(IDS_ERROR_CONNECT,strReplyConnect);
   return -1;
  }

 return POPConverse();
}

#define MAXBUFFER 256
CString CPOPClient::POPReply(int &ErrorCode)
{
 char cResponse[MAXBUFFER+1];
 CString strResponse;
 unsigned length;
 ErrorCode = 0;
       
 do
  {
   length = m_pSocket->Receive(cResponse,MAXBUFFER);
 
   if (bCanceling)
     {
      ErrorCode = -2;
      return "-ERR Canceling request from user";
   	 }

   if (length==SOCKET_ERROR)
     {
      ErrorCode = SOCKET_ERROR;
      return "-ERR Socket Receiving error";
   	 }        
   	 
  	 
   cResponse[length]='\0';
   strResponse += cResponse;
  }
 while (strResponse.Right(2)!="\r\n");

 Log(strResponse);
 return strResponse;
}



CString CPOPClient::POPReply(int &ErrorCode,unsigned long FullSize,unsigned long CurrentSize)
{
 char cResponse[MAXBUFFER+1];
 CString strResponse;
 unsigned length;
 ErrorCode = 0;
       
 do
  {
   length = m_pSocket->Receive(cResponse,MAXBUFFER);
   // Progress(IDS_PROGRESS_MAIL,30+((65*(strResponse.GetLength()-CurrentSize))/FullSize));
   if (bCanceling)
     {
      ErrorCode = -2;
      return "-ERR Canceling request from user";
   	 }

   if (length==SOCKET_ERROR)
     {
      ErrorCode = SOCKET_ERROR;
      return "-ERR Socket Receiving error";
   	 }        
   	 
  	 
   cResponse[length]='\0';
   strResponse += cResponse;
  }
 while ((length==MAXBUFFER)||(strResponse.Right(2)!="\r\n"));

 Log(strResponse);
 return strResponse;
}


int CPOPClient::POPGetSizes(unsigned long* sizes,unsigned messages)
{
 int ErrorCode;

 if (m_pSocket==NULL)
  return -1;

 char sbuffer[20];
 sprintf(sbuffer,"LIST\r\n");
 Log(sbuffer);
 if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return -1;
   }
  if (bCanceling) return -1;

  CString strList = POPReply(ErrorCode);
  if (strList.Left(3)!="+OK")
   {
    SocketError(IDS_ERROR_FROM_POP_SERVER,strList);
    return -1;
   }

  while (strList.Right(5)!="\r\n.\r\n")
  {
   strList += POPReply(ErrorCode);
   if (ErrorCode!=0)
    {
     if (ErrorCode==SOCKET_ERROR)
      SocketError(IDS_ERROR_GETTING_HEADERS,"");
     return -1; 
  	}
  }

 CString strRest1 = strList.Right(strList.GetLength()-strList.Find("\n"));
 CString strRest2;

 for (unsigned index=0;index<messages;index++)
  {
   strRest2 = strRest1.Right(strRest1.GetLength()-strRest1.Find(" "));
   sizes[index] = atol(strRest2.Left(strRest2.Find("\n")));
   strRest1 = strRest2.Right(strRest2.GetLength()-strRest2.Find("\n"));
  }

 return 1;
}


void CPOPClient::POPDisconnect(int iMsgNb)
{
 if (m_pSocket)
  {
   if (!bCanceling)
    {
	 Log("QUIT");
     m_pSocket->Send("QUIT\r\n",6);
   	}
   m_pSocket->Close();
   delete m_pSocket;
   m_pSocket = NULL;
  }

 if (iMsgNb==-1)
  AfxGetMainFrame()->MailCheckEnd1(DD_ERROR);
 else
  AfxGetMainFrame()->MailCheckEnd1(DD_NORMAL,iMsgNb);
}



/* ======================================================================== */
/* POP Converse */
/* ======================================================================== */
int CPOPClient::POPConverse()
{
  int ErrorCode = 0;
  char sbuffer[BUFFERSIZE];

  sprintf(sbuffer, "USER %s\r\n", AfxGetConfig()->m_csPOPUsername.GetBuffer(0));
  Log(sbuffer);

  if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return -1;
   }

  if (bCanceling) return -1;

  CString strReplyUSER = POPReply(ErrorCode);
 
  if (strReplyUSER.Left(3)!="+OK")
   {
    if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
     {
	  SocketError(IDS_ERROR_SEND,"");
	  return -1;
     }

    if (bCanceling) return -1;

    strReplyUSER = POPReply(ErrorCode);
  
    if (strReplyUSER.Left(3)!="+OK")
     {
      SocketError(IDS_ERROR_USER,strReplyUSER);
      return -1;
     }
   }

  sprintf(sbuffer,"PASS %s\r\n",AfxGetConfig()->m_csPOPPassword.GetBuffer(0));
  Log("PASS ********");
  
  if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return -1;
   }

  if (bCanceling) return -1;

 
  CString strReplyPASS = POPReply(ErrorCode);
  if (strReplyPASS.Left(3)!="+OK")
   {
    SocketError(IDS_ERROR_PASS,strReplyPASS);
	return -1;
   }
 
  Log("STAT");
  if (m_pSocket->Send("STAT\r\n",6)==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return -1;
   }
   if (bCanceling) return -1;
 
  CString strReplySTAT = POPReply(ErrorCode);
  if (strReplySTAT.Left(3)!="+OK")
  {
   SocketError(IDS_ERROR_STAT,strReplySTAT);
   return -1;
  }

 int posstart = 4;
 int posend = 4;

 while ((strReplySTAT.GetLength()!=posend)&&(strReplySTAT.GetAt(posend)!=' '))
  posend++;
 return	atoi(strReplySTAT.Mid(posstart,posend-posstart));
}

 


int CPOPClient::GetMessageHeader(unsigned int number,CMail& mail)
{
 char sbuffer[20];
 int ErrorCode = 0;

 if (bCanceling) return -1;

 sprintf(sbuffer,"TOP %d 5\r\n",number);
 Log(sbuffer);
 if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return -1;
   }
 if (bCanceling) return -1;
 
 CString strReplyMsg = POPReply(ErrorCode);
 if (strReplyMsg.Left(3)!="+OK")
   {
    SocketError(IDS_ERROR_FROM_POP_SERVER,strReplyMsg);
   	return -1;
   }
 
 int posstart = 4;
 int posend = 4;
 
 while ((strReplyMsg.GetLength()!=posend)&&(strReplyMsg.GetAt(posend)!=' '))
   posend++;
 unsigned long length = atol(strReplyMsg.Mid(posstart,posend-posstart));
 DEBUG4(("Bytes to read %u",length));

 while (strReplyMsg.Right(5)!="\r\n.\r\n")
  {
   strReplyMsg += POPReply(ErrorCode);
   if (ErrorCode!=0)
    {
     if (ErrorCode==SOCKET_ERROR)
      SocketError(IDS_ERROR_GETTING_HEADERS,"");
     return -1; 
  	}
  }
 
 if (strReplyMsg.Find("\r\nStatus:")==-1)
  {
   length+=10;
  }

 CString strDate = "No Date";
 CString strSubject = "No Subject";
 CString strAuthor = "No Author";


 unsigned nPosDateBegin,nPosAuthorBegin,nPosSubjectBegin;
 unsigned nPosDateEnd,nPosAuthorEnd,nPosSubjectEnd;
 nPosDateBegin = strReplyMsg.Find("\r\nDate:");
 nPosAuthorBegin = strReplyMsg.Find("\r\nFrom:");
 nPosSubjectBegin = strReplyMsg.Find("\r\nSubject:");

 if (nPosDateBegin!=-1)
  {
   for (nPosDateEnd=nPosDateBegin+6;nPosDateEnd<(unsigned)strReplyMsg.GetLength();nPosDateEnd++)
    if ((strReplyMsg.GetAt(nPosDateEnd)=='\r')||(strReplyMsg.GetAt(nPosDateEnd)=='\n'))
    break;
   strDate =  strReplyMsg.Mid(nPosDateBegin+8,nPosDateEnd-nPosDateBegin-8);
  }

 if (nPosAuthorBegin!=-1)
  {
   for (nPosAuthorEnd=nPosAuthorBegin+6;nPosAuthorEnd<(unsigned)strReplyMsg.GetLength();nPosAuthorEnd++)
    if ((strReplyMsg.GetAt(nPosAuthorEnd)=='\r')||(strReplyMsg.GetAt(nPosAuthorEnd)=='\n'))
     break;
   strAuthor = strReplyMsg.Mid(nPosAuthorBegin+8,nPosAuthorEnd-nPosAuthorBegin-8);
  }
  			  
 if (nPosSubjectBegin!=-1)
  {
   for (nPosSubjectEnd=nPosSubjectBegin+11;nPosSubjectEnd<(unsigned)strReplyMsg.GetLength();nPosSubjectEnd++)
    if ((strReplyMsg.GetAt(nPosSubjectEnd)=='\r')||(strReplyMsg.GetAt(nPosSubjectEnd)=='\n'))
     break;
   strSubject =	strReplyMsg.Mid(nPosSubjectBegin+11,nPosSubjectEnd-nPosSubjectBegin-11);
  }
  			
  mail.SetMail(strAuthor,strSubject,strDate,strReplyMsg,length);
  return 1;
} 



BOOL CPOPClient::DeleteMail(int mailnb)
{
 char sbuffer[20];
 int ErrorCode = 0;

 if (bCanceling) return FALSE;

 sprintf(sbuffer,"DELE %d\r\n",mailnb);
 Log(sbuffer);
 if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return FALSE;
   }
 
 if (bCanceling) return FALSE;
 
 CString strReplyMsg = POPReply(ErrorCode);
 if (strReplyMsg.Left(3)!="+OK")
   {
    SocketError(IDS_ERROR_FROM_POP_SERVER,strReplyMsg);
   	return FALSE;
   }
 
 return TRUE;
}


BOOL CPOPClient::ViewMail(int mailnb,CString& strMail,unsigned long size)
{
 char sbuffer[20];
 int ErrorCode = 0;

 if (bCanceling) return FALSE;

 sprintf(sbuffer,"TOP %d 200\r\n",mailnb);
 Log(sbuffer);
 if (m_pSocket->Send(sbuffer,strlen(sbuffer))==SOCKET_ERROR)
   {
	SocketError(IDS_ERROR_SEND,"");
	return FALSE;
   }
 
 if (bCanceling) return FALSE;
 
 strMail = POPReply(ErrorCode,size,0);
 if (strMail.Left(3)!="+OK")
   {
    SocketError(IDS_ERROR_FROM_POP_SERVER,strMail);
   	return FALSE;
   }
 
 while (strMail.Right(5)!="\r\n.\r\n")
  {
   Progress(IDS_PROGRESS_MAIL,30+(65*(size-strMail.GetLength()))/size);
   strMail += POPReply(ErrorCode,size,strMail.GetLength());
   if (ErrorCode!=0)
    {
     if (ErrorCode==SOCKET_ERROR)
      SocketError(IDS_ERROR_GETTING_HEADERS,"");
     return FALSE; 
  	}
  }
 return TRUE;
}



/* ======================================================================== */
/* SocketError
/* ======================================================================== */
void CPOPClient::SocketError(UINT MessageCode,CString strReturn)
{
 // Hang Up DialUp Networking to be sure we do not stay online
 AfxGetMainFrame()->DialUpDisconnect(DD_ERROR);
 
 if (strReturn.GetLength()==0)
  {
   CString strmessage;
 
   if (!strmessage.LoadString(MessageCode))
    AfxThrowResourceException();

   Log(strmessage);
  }
 else
  Log(strReturn);

 if (bCanceling==FALSE)
    {
	 AfxGetMainFrame()->MailCheckEnd1(DD_ERROR);
	 AfxGetMainFrame()->SendMessage(WM_SET_MESSAGE,MessageCode,2);
    }
 }


void CPOPClient::Log(CString message)
{
 if (AfxGetConfig()->m_bLOGLogging)
  {
   CTime time = CTime::GetCurrentTime();
   CString strTime = time.Format("%m/%d/%y %H:%M:%S");

   ofstream logfile;
   logfile.open(AfxGetConfig()->m_csLOGFilename, ios::out | ios::app);
   
   if (logfile.is_open())
    {
	 if (message.GetLength())
      for (int i=0;i<message.GetLength();i++)
	   {
        if ((message.GetAt(i)=='\n')||(message.GetAt(i)=='\r'))
		 message.SetAt(i,' ');
	   }
  	 if (AfxGetConfig()->m_bLOGVerbose)
  	  logfile << strTime << " : " << message << "\n";
  	 else
	  logfile << strTime << " : " << message.Left(50) << "\n";
   	 logfile.close();
  	}
  }
};


void CPOPClient::Progress(UINT MessageCode,unsigned Progress)
{
  AfxGetMainFrame()->SendMessage(WM_NOTIFY_PROGRESS,MessageCode,Progress);
}



