/*********************************************************************************
 *																				
 *   e-Mail Notify - Internet e-Mail Notifier
 *   Copyright (C) 1996-1998 Ludovic Dubost <ldubost@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 **********************************************************************************/

#include "stdafx.h"

#include <afxpriv.h>
#include <stddef.h>

#include "email.h"
#include "mainfrm.h"
#include "popsock.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#ifdef WIN32
#define DELETE_EXCEPTION(e) do { e->Delete(); } while (0)
#endif


CPOPSocket::CPOPSocket() : CSocket()
{
}


BOOL CPOPSocket::OnMessagePending()
{
 MSG Msg;

 if (::PeekMessage(&Msg,AfxGetMainFrame()->GetSafeHwnd(),0,0,PM_REMOVE))
       {
#ifndef WIN95
	 if ((Msg.message!=WM_SOCKET_NOTIFY)
			&&(Msg.message!=WM_SOCKET_DEAD)
			&&(Msg.message!=WM_TIMER))
		{
	    ::TranslateMessage(&Msg);
        ::DispatchMessage(&Msg);
		}
#else
	    ::TranslateMessage(&Msg);
        ::DispatchMessage(&Msg);
#endif
	 }
 
 if ((Msg.message==WM_PAINT)||(Msg.message==WM_IDLEUPDATECMDUI))
  return FALSE; 
 else
  return TRUE; 
}

#if defined(WINNT35) || defined(WIN32S)
BOOL CPOPSocket::PumpMessages(UINT uStopFlag)
{
	// The same socket better not be blocking in more than one place.
	ASSERT(m_pbBlocking == NULL);

	AFX_THREAD_STATE* pThreadState = AfxGetThreadState();

	ASSERT(pThreadState->m_hSocketWindow != NULL);

	BOOL bBlocking = TRUE;
	
	m_pbBlocking = &bBlocking;
	CWinThread* pThread = AfxGetThread();

   
	UINT nTimerID = ::SetTimer(NULL, 0xff00, m_nTimeOut, NULL);


	if (nTimerID == 0)
		AfxThrowResourceException();

	while (bBlocking)
	{
		TRY
		{
			MSG msg;
			if (::PeekMessage(&msg, pThreadState->m_hSocketWindow,
				WM_SOCKET_NOTIFY, WM_SOCKET_DEAD, PM_REMOVE))
			{
				if (msg.message == WM_SOCKET_NOTIFY && (SOCKET)msg.wParam == m_hSocket)
				{
					if (WSAGETSELECTEVENT(msg.lParam) == FD_CLOSE)
					{
						break;
					}
					if (WSAGETSELECTEVENT(msg.lParam) == uStopFlag)
					{
						if (uStopFlag == FD_CONNECT)
							m_nConnectError = WSAGETSELECTERROR(msg.lParam);
						break;
					}
				}
				//  This is a Bug cause in the CSocket implementation
				//  It causes dead socket messages to be processed
				//  if ((SOCKET)msg.wParam == m_hSocket && msg.lParam != 0)
				//	CSocket::AuxQueueAdd(msg.message, msg.wParam, msg.lParam);
			}
			else if (::PeekMessage(&msg, NULL, WM_TIMER, WM_TIMER, PM_REMOVE))
			{
			   if (msg.wParam == nTimerID)
				{
					break;
				}
				else
				{
				  ::TranslateMessage(&msg);
   				  ::DispatchMessage(&msg);
   				} 
			}
			else if ((::PeekMessage(&msg, AfxGetMainFrame()->GetSafeHwnd(), 0, 0, PM_NOREMOVE))
			         &&(msg.message!=WM_SOCKET_NOTIFY)
			         &&(msg.message!=WM_SOCKET_DEAD)
			         &&(msg.message!=WM_TIMER)
					)
			 {     
			   if (OnMessagePending())
			    {	
			     // allow user-interface updates
			     pThread->OnIdle(-1);
			    }
			   else
			    {
				// no work to do -- allow CPU to sleep
				WaitMessage();
			    } 
			 }   
		}
		CATCH_ALL(e)
		{
			TRACE0("Error: caught exception in PumpMessage - continuing.\n");
			DELETE_EXCEPTION(e);
		}
		END_CATCH_ALL
	}

	::KillTimer(NULL, nTimerID);

	if (!bBlocking)
	{
		WSASetLastError(WSAEINTR);
		return FALSE;
	}
	m_pbBlocking = NULL;

	::PostMessage(pThreadState->m_hSocketWindow,WM_SOCKET_NOTIFY,0,0);

 	return TRUE;
}
#endif
#ifdef WIN16
BOOL CPOPSocket::PumpMessages(UINT uStopFlag)
{
	// The same socket better not be blocking in more than one place.
	ASSERT(m_pbBlocking == NULL);

	BOOL bBlocking = TRUE;
	m_pbBlocking = &bBlocking;
	CWinApp* pApp = AfxGetApp();

	UINT nTimerID = ::SetTimer(NULL,0xff00,m_nTimeOut,NULL);

	if (nTimerID == 0)
		AfxThrowResourceException();

	while (bBlocking)
	{
		TRY
		{
			MSG msg;
			if (::PeekMessage(&msg, NULL,
				WM_SOCKET_NOTIFY, WM_SOCKET_DEAD, PM_REMOVE))
			{
			 if (msg.message == WM_SOCKET_NOTIFY && (SOCKET)msg.wParam == m_hSocket)
				{
					if (WSAGETSELECTEVENT(msg.lParam) == FD_CLOSE)
					{
						break;
					}
					if (WSAGETSELECTEVENT(msg.lParam) == uStopFlag)
					{
						if (uStopFlag == FD_CONNECT)
							m_nConnectError = WSAGETSELECTERROR(msg.lParam);
						break;
					}
				}
			//	if (msg.wParam != 0 || msg.lParam != 0)
			//		CSocket::AuxQueueAdd(msg.message, msg.wParam, msg.lParam);
			}
			else if (::PeekMessage(&msg, NULL, WM_TIMER, WM_TIMER, PM_REMOVE))
			{
				if (msg.wParam == nTimerID)
				{
					break;
				}
				else
				{
				  ::TranslateMessage(&msg);
   				  ::DispatchMessage(&msg);
   				} 
			}
			else if ((::PeekMessage(&msg, AfxGetMainFrame()->GetSafeHwnd(), 0, 0, PM_NOREMOVE))
				   &&(msg.message!=WM_SOCKET_NOTIFY)
			       &&(msg.message!=WM_SOCKET_DEAD)
			       &&(msg.message!=WM_TIMER)
			       )
			 {      
			   if (OnMessagePending())
			    {	
			     // allow user-interface updates
			     pApp->OnIdle(-1);
			    }
			   else
			    {
				// no work to do -- allow CPU to sleep
				WaitMessage();
			    } 
			 }   
		}
		CATCH_ALL(e)
		{
			TRACE0("Error: caught exception in PumpMessage - continuing.\n");
		}
		END_CATCH_ALL
	}

	::KillTimer(NULL, nTimerID);

	if (!bBlocking)
	{
		WSASetLastError(WSAEINTR);
		return FALSE;
	}
	m_pbBlocking = NULL;
 	return TRUE;
}
#endif
 

